import { __assign } from "tslib";
import { httpFetch } from './fetch';
/**
 * Creates new Pilet API extensions for fetch.
 * @param config The custom fetch configuration, if any.
 */
export function createFetchApi(config) {
    if (config === void 0) { config = {}; }
    return function (context) {
        return {
            fetch: function (path, options) {
                if (options === void 0) { options = {}; }
                var originalHeaders = options.headers || {};
                var headerPromises = [];
                context.emit('before-fetch', {
                    headers: originalHeaders,
                    method: options.method || 'GET',
                    target: path,
                    setHeaders: function (headers) {
                        if (headers) {
                            headerPromises.push(headers);
                        }
                    },
                });
                return Promise.all(headerPromises)
                    .then(function (newHeaders) {
                    var headers = newHeaders.reduce(function (obj, header) {
                        if (typeof header === 'object' && header) {
                            return __assign(__assign({}, obj), header);
                        }
                        return obj;
                    }, originalHeaders);
                    return __assign(__assign({}, options), { headers: headers });
                })
                    .then(function (options) { return httpFetch(config, path, options); });
            },
        };
    };
}
//# sourceMappingURL=create.js.map