"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFetchApi = void 0;
var tslib_1 = require("tslib");
var fetch_1 = require("./fetch");
/**
 * Creates new Pilet API extensions for fetch.
 * @param config The custom fetch configuration, if any.
 */
function createFetchApi(config) {
    if (config === void 0) { config = {}; }
    return function (context) {
        return {
            fetch: function (path, options) {
                if (options === void 0) { options = {}; }
                var originalHeaders = options.headers || {};
                var headerPromises = [];
                context.emit('before-fetch', {
                    headers: originalHeaders,
                    method: options.method || 'GET',
                    target: path,
                    setHeaders: function (headers) {
                        if (headers) {
                            headerPromises.push(headers);
                        }
                    },
                });
                return Promise.all(headerPromises)
                    .then(function (newHeaders) {
                    var headers = newHeaders.reduce(function (obj, header) {
                        if (typeof header === 'object' && header) {
                            return tslib_1.__assign(tslib_1.__assign({}, obj), header);
                        }
                        return obj;
                    }, originalHeaders);
                    return tslib_1.__assign(tslib_1.__assign({}, options), { headers: headers });
                })
                    .then(function (options) { return fetch_1.httpFetch(config, path, options); });
            },
        };
    };
}
exports.createFetchApi = createFetchApi;
//# sourceMappingURL=create.js.map